import uuid
from typing import Union

from elemental_tools.config import config_initializer
from elemental_tools.constants import ref_length
from elemental_tools.logger import Logger
from elemental_tools.settings import SettingController, SettingSchema
from elemental_tools.system import generate_reference

from elemental_tools.tools import get_package_name
from elemental_tools.exceptions import UndefinedTable

config = config_initializer()

__logger__ = Logger(app_name=config.app_name, owner=get_package_name(__name__)).log

_s_controller = SettingController()
_autogenerated_root_ref = generate_reference()

internal_roles = ["attendant", "manager"]
url_google_sheets = "https://docs.google.com/spreadsheets/d/"
url_google_drive = "https://drive.google.com/drive/folders/"
url_wpp_me = "https://wa.me/"


def root_ref():
    return _s_controller.root_ref.get(sub=None, default=_autogenerated_root_ref)


class DefaultTax:
    tax = _s_controller.crypto_default_taxes.get(sub=root_ref())

    def __init__(self, sub=root_ref()):
        __logger__('info', 'Retrieving Tax Information...')
        self.tax = _s_controller.crypto_default_taxes.get(sub=sub)
        __logger__('success', f'The defined tax is: {self.tax}')

    def __call__(self):
        self.__init__()
        return self.tax


default_tax = DefaultTax(root_ref())

